/*
 * Audible_Dice_Tower_3 Digit_LED.c
 *
 * Created: 5/6/2020 7:51:06 PM
 * Author : johna
 *
 * Copyright (C) 2020 by John Anderson <racerxr650r@gmail.com>
 *
 * Permission to use, copy, modify, and/or distribute this software for any 
 * purpose with or without fee is hereby granted.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES 
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN 
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */ 

#include <avr/io.h>
#include <stdlib.h>
#include <stdbool.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include <util/delay_basic.h>

// CPU Pinout *****************************************************************
//
//            +-------------U--------------+
//            |         ATMega 328P        |
//     ISP(6)-+PC6/RESET       ADC5/SCL/PC5+-CNT-SW
//   Digit1-A-+PD0/RXD         ADC4/SDA/PC4+-CNT-DATA
//   Digit1-B-+PD1/TXD             ADC3/PC3+-CNT-CLK
//   Digit1-C-+PD2/INT0            ADC2/PC2+-DIE-SW
//   Digit1-D-+PD3/OC2B/INT1       ADC1/PC1+-DIE-DATA
//   Digit2-A-+PD4/XCL/T0          ADC0/PC0+-DIE-CLK
//    	     -+VCC                      GND+-
//           -+GND                     AREF+-
//       D100-+PB6/XTAL1/TOSC1         AVCC+-
//     OVRFLW-+PB7/XTAL2/TOSC2      SCK/PB5+-ISP(5)
//   Digit2-B-+PD5/OC0B/T1         MISO/PB4+-ISP(4)
//   Digit2-C-+PD6/OC0A/AIN0  OC2A/MOSI/PB3+-SPEAKER/ISP(2)
//   Digit2-D-+PD7/AIN1         OC1B/SS/PB2+-N/A
//     RESULT-+PB0/CLK0/ICP1       OC1A/PB1+-ROLL
//            +----------------------------+
 
// Program constants **********************************************************
#define BLANK_OUTPUT				00
#define DISPLAY_BLANK_TIME			1000
#define DISPLAY_CONFIG_TIME			500

// Busy Delay function ********************************************************
#define F_CPU_KHZ 8000  // CPU clock speed in kHz
#define LOOPS_PER_MS (uint16_t)(F_CPU_KHZ/4)

// Program data types *********************************************************
typedef enum {State_Init, State_Idle, State_Update_Dice, State_Speak_Dice, State_Roll, State_Roll_Result}State;
typedef enum {D2, D4, D6, D8, D10, D12, D20, D100, DIE_TYPES}DieType;
	
const int DieSides[] = {2,4,6,8,10,12,20,100};
	
const int Character[] = {0b00111111,	// '0'
						 0b00000110,	// '1'
						 0b01011011,    // '2'
						 0b01001111,	// '3'
						 0b01100110,	// '4'
						 0b01101101,	// '5'
						 0b01111101,	// '6'
						 0b00000111,	// '7'
						 0b01111111,	// '8'
						 0b01101111,	// '9'
						 0b01110111,	// 'A'
						 0b01111100,	// 'b'
						 0b01011000,	// 'c'
						 0b01011110,	// 'd'
						 0b01111001,	// 'E'
						 0b01110001,	// 'F'
						 0b00000000,	// ' '
						 0b01000000,	// '-'
						 0b01001000};	// '='
						 
#define CHAR_BLANK	0x10
#define CHAR_DASH	0x11
#define CHAR_EQUALS 0x12

// Busy wait delay function ***************************************************
void delay_ms(uint16_t ms)
{
	while(ms--)
		_delay_loop_2(LOOPS_PER_MS);
}

// I/O Constants **************************************************************
/* Port B Usage ******************************
Port	Config		Device
PB0		DO			Result Indicator LED
PB1		DI			Roll Button
PB2		N/A			N/A
PB3		DO/MOSI		Speaker output / In circuit programming MOSI
PB4		DO/MISO		In circuit programming MISO
PB5		DO/SCK		In circuit programming SCK
PB6		DO			Nixie 1 Hundred
PB7		DO			Nixie Overflow indicator
*********************************************/
#define RESULT_LED		0b00000001
#define ROLL_BUTTON		0b00000010
#define EXPLODING_DIE	0b00000100
#define SPEAKER			0b00001000
#define ISP_MOSI		0b00001000
#define ISP_MISO		0b00010000
#define ISP_SCK			0b00100000
#define ISP_SCK			0b00100000
#define NIX_D3_B0		0b01000000
#define NIX_OVRFLW		0b10000000
/* Port C Usage ******************************
Port	Config		Device
PC0		DI			Die Select Clock
PC1		DI			Die Select Data
PC2		DI			Die Select Button
PC3		DI			Die Count Clock
PC4		DI			Die Count Data
PC5		DI			Die Count Button
PC6		RESET		ISP-RESET
*********************************************/
#define DIE_SELECT_CLK	0b00000001
#define DIE_SELECT_DATA	0b00000010
#define DIE_SELECT_BTTN	0b00000100
#define DIE_COUNT_CLK	0b00001000
#define DIE_COUNT_DATA	0b00010000
#define DIE_COUNT_BTTN	0b00100000
#define ISP_RESET		0b01000000
/* Port D Usage ******************************
Port	Config		Device
PD0		D0			Nixie Digit 1 Bit 0
PD1		D0			Nixie Digit 1 Bit 1
PD2		D0			Nixie Digit 1 Bit 2
PD3		D0			Nixie Digit 1 Bit 3
PD4		D0			Nixie Digit 2 Bit 0
PD5		D0			Nixie Digit 2 Bit 1
PD6		D0			Nixie Digit 2 Bit 2
PD7		D0			Nixie Digit 2 Bit 3
*********************************************/
#define NIX_D1_B0	0b00000001
#define NIX_D1_B1	0b00000010
#define NIX_D1_B2	0b00000100
#define NIX_D1_B3	0b00001000
#define NIX_D2_B0	0b00010000
#define NIX_D2_B1	0b00100000
#define NIX_D2_B2	0b01000000
#define NIX_D2_B3	0b10000000

// Setup the IO pins **********************************************************
void initIO()
{
	// Set Port B pins to outputs
	DDRB = RESULT_LED | NIX_D3_B0 | NIX_OVRFLW;
	// Set Port B pins to inputs
	DDRB &= ~(ROLL_BUTTON | EXPLODING_DIE);
	// Enable pull ups on Port B
	PORTB = ROLL_BUTTON | EXPLODING_DIE;
	// Set Port C pins to input (no outputs)
	DDRC &= ~(DIE_SELECT_CLK | DIE_SELECT_DATA | DIE_SELECT_BTTN | DIE_COUNT_CLK | DIE_COUNT_DATA | DIE_COUNT_BTTN);
	// Enable pull ups on Port C
	//PORTC = DIE_SELECT_CLK | DIE_SELECT_DATA | DIE_SELECT_BTTN | DIE_COUNT_CLK | DIE_COUNT_DATA | DIE_COUNT_BTTN;
	// Set Port D pins to outputs
	DDRD = NIX_D1_B0 | NIX_D1_B1 | NIX_D1_B2 | NIX_D1_B3 | NIX_D2_B0 | NIX_D2_B1 | NIX_D2_B2 | NIX_D2_B3;
	
	// Enable interrupts on Port C pins connected to the 5 way input switch
	// So they can awake the CPU from sleep mode
	PCICR |= (1<<PCIE1 | 1<<PCIE0);
	PCMSK1 |= (DIE_SELECT_CLK | DIE_SELECT_BTTN | DIE_COUNT_CLK | DIE_COUNT_BTTN);
	PCMSK0 |= ROLL_BUTTON;
}

// I/O interface functions ****************************************************
// Get the user intput **********************************************************
#define INPUT_TYPE_DOWN			0b00000001
#define INPUT_TYPE_UP			0b00000010
#define INPUT_TYPE_BUTTON		0b00000100
#define INPUT_COUNT_DOWN		0b00001000
#define INPUT_COUNT_UP			0b00010000
#define INPUT_COUNT_BUTTON		0b00100000
#define INPUT_ROLL				0b01000000

volatile uint8_t inputFlag=0, portBStatus = 0b00000010, portBFlag = 0, portCStatus = 0b00111111, portCFlag = 0;

// Interrupt handler for Port B input change events
ISR(PCINT0_vect)
{
	delay_ms(1); // Debounce
	// Update the pin change flag register and pin status register
	portBFlag = PINB ^ portBStatus;
	portBStatus = PINB;
	
	// If roll button press...
	if((portBFlag & ROLL_BUTTON) && ~(portBStatus & INPUT_ROLL))
		inputFlag |= INPUT_ROLL;
	//delay_ms(100); // Debounce
}
// Interrupt handler for Port C input change events
ISR(PCINT1_vect)
{
	delay_ms(1);
	// Update the pin change flag register and pin status register
	portCFlag = PINC ^ portCStatus;
	portCStatus = PINC;
	
	// If dice type select rotary encoder clock signal falling edge...
	if((portCFlag & DIE_SELECT_CLK) && !(portCStatus & DIE_SELECT_CLK))
	{
		if(portCStatus & DIE_SELECT_DATA)
			inputFlag |= INPUT_TYPE_UP;
		else
			inputFlag |= INPUT_TYPE_DOWN;
	}

	// If dice type rotary encoder button pressed...
	if((portCFlag&DIE_SELECT_BTTN) && (portCStatus&DIE_SELECT_BTTN))
		inputFlag |= INPUT_TYPE_BUTTON;

	// If dice count select rotary encoder clock signal falling edge...
	if((portCFlag & DIE_COUNT_CLK) && !(portCStatus & DIE_COUNT_CLK))
	{
		if(portCStatus & DIE_COUNT_DATA)
			inputFlag |= INPUT_COUNT_UP;
		else
			inputFlag |= INPUT_COUNT_DOWN;
	}
	
	// If dice count select rotary encoder button pressed...
	if((portCFlag&DIE_COUNT_BTTN) && (portCStatus&DIE_COUNT_BTTN))
		inputFlag |= INPUT_COUNT_BUTTON;
}

// Check for inputs in the input flag register
bool chkInput()
{
	// If an input is flagged...
	if(inputFlag)
	{
		// Return TRUE (1)
		return(true);
	}
	// Else no inputs flagged...
	else
	{
		// Return FALSE (0)
		return(false);
	}
}

// Clear the input flag register
void clrInput()
{
	inputFlag = 0;
}

// Get return the current input flag register and clear it
uint8_t getInput()
{
	// Return the current value of the inputFlag register
	uint8_t ret = inputFlag;
	
	// Clear the inputFlag register
	inputFlag = 0;
	
	return(ret);
}

// Get the status of the Exploding Dice setting
bool getExplodingDice()
{
	bool ret = false;
	
	if((PINB&EXPLODING_DIE)==0)
		ret = true;
		
	return(ret);
}

// Display Output *************************************************************
// Result Indicator ***********************************************************
void resultIndicator(bool state)
{
	if(state)
		PORTB |= RESULT_LED;
	else
		PORTB &= ~RESULT_LED;
}

// One Hundred Digit Indicator ************************************************
void hundredIndicator(bool state)
{
	if(state)
		PORTB |= NIX_D3_B0;
	else
		PORTB &= ~NIX_D3_B0;
}

// Overflow Indicator *********************************************************
void overflowIndicator(bool state)
{
	if(state)
		PORTB |= NIX_OVRFLW;
	else
		PORTB &= ~NIX_OVRFLW;
}

// Output a two digit decimal number on the Nixie Tube display ****************
void output(int out)
{
	// Calculate the ones digit
	int outputByte = out%10;
	// Calculate the tens digit
	out /= 10;
	outputByte |= (out%10)<<4;
	// Write to the output port
	PORTD = outputByte;
	
	// Calculate the hundreds digit
	out /= 10;
	// Set hundreds digit and overflow indicators accordingly...
	switch(out)
	{
		case 0:
			hundredIndicator(false);
			overflowIndicator(false);
			break;
		case 1:
			hundredIndicator(true);
			overflowIndicator(false);
			break;
		default:
			hundredIndicator(true);
			overflowIndicator(true);
			break;
	}
}

// Sound output functions *****************************************************
// Include PCM sound images ***************************************************
#include "dt_numbers_7.h"
// Sound globals, handlers, and functions *************************************
#define IS_SOUND_PLAYING gTmrCntr

#define SND_HIGH	0x03
#define SND_MEDIUM	0x07
#define SND_LOW		0x0f

#define TONE_C8		4186
#define TONE_B7		3951
#define TONE_As7	3729
#define TONE_A7		3520
#define TONE_Gs7	3322
#define TONE_G7		3136
#define TONE_Fs7	2960
#define TONE_F7		2794
#define TONE_E7		2637
#define TONE_Ds7	2489
#define TONE_D7		2349
#define TONE_Cs7	2217
#define TONE_C7		2093
#define TONE_B6		1976
#define TONE_As6	1865
#define TONE_A6		1760
#define TONE_Gs6	1661
#define TONE_G6		1568
#define TONE_F6		1397
#define TONE_E6		1319
#define TONE_Ds6	1245
#define TONE_D6		1175
#define TONE_Cs6	1109
#define TONE_C6		1047
#define TONE_B5		988
#define TONE_As5	932
#define TONE_A5		880
#define TONE_Gs5	831
#define TONE_G5		784
#define TONE_Fs5	740
#define TONE_F5		698
#define TONE_E5		659
#define TONE_Ds5	622
#define TONE_D5		587
#define TONE_Cs5	554
#define TONE_C5		523
#define TONE_B4		494
#define TONE_As4	466
#define TONE_A4		440
#define TONE_Gs4	415
#define TONE_G4		392
#define TONE_Fs4	370
#define TONE_F4		349
#define TONE_E4		330
#define TONE_Ds4	311
#define TONE_D4		294
#define TONE_Cs4	277
#define TONE_C4		262
#define TONE_B3		247
#define TONE_As3	233
#define TONE_A3		220
#define TONE_Gs3	208
#define TONE_G3		196
#define TONE_Fs3	185
#define TONE_F3		175
#define TONE_E3		165
#define TONE_Ds3	156
#define TONE_D3		147
#define TONE_Cs3	139
#define TONE_C3		131
#define TONE_B2		123
#define TONE_As2	117
#define TONE_A2		110
#define TONE_Gs2	104
#define TONE_G2		98
#define TONE_Fs2	92
#define TONE_F2		87
#define TONE_E2		82
#define TONE_Ds2	78
#define TONE_D2		74
#define TONE_Cs2	69
#define TONE_C2		65
#define TONE_B1		62
#define TONE_As1	58
#define TONE_A1		55
#define TONE_Gs1	52
#define TONE_G1		49
#define TONE_Fs1	46
#define TONE_F1		44
#define TONE_E1		41
#define TONE_Ds1	39
#define TONE_D1		37
#define TONE_Cs1	35
#define TONE_C1		33

volatile uint32_t gTmrCntr, gTmrEnd;
volatile uint16_t gSndCnt, gSndEnd, gRleCnt;
volatile uint8_t  gRate;
volatile const uint8_t  *gSound;

#define CONTROL_REG_A	TCCR2A		// Control register A
#define CONTROL_REG_B	TCCR2B		// Control register B
#define COUNT_REG		TCNT2		// Counter register
#define MASK_REG		TIMSK2		// Mask Register
#define FLAG_REG		TIFR2		// Flag Register
#define COMPARE_REG_A	OCR2A		// Compare register A
#define COMPARE_REG_B	OCR2B		// Compare register B
#define IO_DIRECTION	DDRB		// IO data direction register
#define IO_PORT			PORTB		// IO Port register

// Load the PWM output value
void inline loadPWM(void)
{
	uint8_t byte;
	
	if(gRleCnt)
	{
		--gRleCnt;
		COMPARE_REG_A = 0x80;
	}
	else
	{
		byte = pgm_read_byte_near(&gSound[gSndCnt++]);
		if(byte)
			COMPARE_REG_A = byte;
		else
		{
			gRleCnt = pgm_read_byte_near(&gSound[gSndCnt++])-1;
			COMPARE_REG_A = 0x80;
		}
	}
}

// Enable the PWM/Timer and overflow interrupt
void startPWM(uint16_t freq)
{
	// Zero the Hardware Timer Counter
	COUNT_REG = 0x00;
	// Zero the Software Timer Counter
	gTmrCntr = 1;
	// Set the DDR for the output pin
	IO_DIRECTION |= SPEAKER;
	// If no frequency provided, play a PCM'd sound from flash
	if(!freq)
	{
		// Set the run length encoded counter to zero
		gRleCnt = 0;
		// Set up the Compare Output and Wave Generation modes
		// non-inverting, PWM fast mode
		CONTROL_REG_A = 0x83;
		// Load the first PCM value into the PWM output comparator
		loadPWM();
		// Enable the overflow interrupt on timer 0
		MASK_REG = (1<<TOIE0);
		// Start the timer
		// No pre-scaler
		// Timer frequency = 8 mHz system clock
		CONTROL_REG_B = 0x01;
	}
	else
	{
		
		// Set up the Compare Output and Wave Generation modes
		// Toggle OC0A, CTC mode
		CONTROL_REG_A = 0x42;
		// Calculate the output compare
		// Divide the timer frequency by 2 because the full period requires to compare/overflows
		COMPARE_REG_A = (uint8_t)((uint16_t)31250/(uint16_t)freq);
		// Enable the timer 0 output compare A interrupt
		MASK_REG = 0x02;
		// Start the timer
		// 256 pre-scaler
		// Timer frequency = 8 mHz system clock / 128 prescaler = 62500 Hz
		CONTROL_REG_B = 0x05;
	}
}

// Disable the PWM/Timer and overflow interrupt
void stopPWM(void)
{
	// Stop the timer
//	CONTROL_REG_B = 0;
	// Clear the Compare Output and Wave Generation modes
//	CONTROL_REG_A = 0;
	// Disable the overflow interrupt on timer 0
	MASK_REG = 0;
	// Clear the DDR for the output pin
	IO_DIRECTION &= ~SPEAKER;
	IO_PORT |= SPEAKER;
	// Set the output pin to output
	// Set Port D pins to outputs
//	DDRD = DDRD | SPEAKER;
//	PORTD &= ~SPEAKER;
	// Zero the Sound Counter so a new sound can be played
	gSndCnt = 0;
	// Zero the Software Timer Counter
	gTmrCntr = 0;
}

// Interrupt handler for PCM'd sound output (Timer 2 overflow)
ISR(TIMER2_OVF_vect)
{
	++gTmrCntr;
	
	// Update according to sample rate (High - 8K, Medium - 4K, Low - 2K)
	if(!(gTmrCntr & gRate))
	{
		// Load the PWM compare register with the next PCM value
		loadPWM();
		
		// If this is the end of the sound data, stop the PWM/Timer
		if((gSndCnt > gSndEnd) && (!gRleCnt))
			stopPWM();
	}
}

static int8_t playSound(const uint8_t *sound, uint8_t rate, uint16_t end)
{
	int8_t ret = 0;
	
	// If no sound is currently playing
	if(!IS_SOUND_PLAYING)
	{
		gRate = rate;
		gSound = sound;
		gSndEnd = end;
		
		// Start the PWM/Timer and interrupts
		startPWM(0);
	}
	// There is a sound playing so return ERROR
	else
		ret = -1;
	
	return(ret);
}

static int8_t playSoundWait(const uint8_t *sound, uint8_t rate, uint16_t end)
{
	int8_t ret;
	
	ret = playSound(sound,rate,end);
	while(IS_SOUND_PLAYING);
	
	return(ret);
}

// Arrays containing the PCM sound samples for all the digits/tens
const uint8_t * onesPCM[] = {NULL,one,two,three,four,five,six,seven,eight,nine,ten,eleven,twelve,thirteen,fourteen,fifteen,sixteen,seventeen,eighteen,nineteen};
const unsigned onesSize[] = {0,sizeof(one),sizeof(two),sizeof(three),sizeof(four),sizeof(five),sizeof(six),sizeof(seven),sizeof(eight),sizeof(nine),sizeof(ten),sizeof(eleven),sizeof(twelve),sizeof(thirteen),sizeof(fourteen),sizeof(fifteen),sizeof(sixteen),sizeof(seventeen),sizeof(eighteen),sizeof(nineteen)};
const uint8_t * tensPCM[] = {NULL,NULL,twenty,thirty,four,fifty,six,seven,eight,nine};
const unsigned tensSize[] = {0,0,sizeof(twenty),sizeof(thirty),sizeof(four),sizeof(fifty),sizeof(six),sizeof(seven),sizeof(eight),sizeof(nine)};

void speak(int num)
{
	int ones = num%10, tens = (num/10)%10, hundreds = (num/100)%10;
	
	//StopDisplay();
	
	if(num<1000 && num>0)
	{
		if(hundreds>0)
		{
			playSoundWait(onesPCM[hundreds],SND_HIGH,onesSize[hundreds]);
			playSoundWait(hundred,SND_HIGH,sizeof(hundred));
			delay_ms(10);
		}
		if(tens>1)
		{
			playSoundWait(tensPCM[tens],SND_HIGH,tensSize[tens]);
			if(tens==4 || tens==6 || tens==7 || tens==8 || tens==9)
				playSoundWait(D,SND_HIGH,sizeof(D));
			delay_ms(10);
		}
		else if(tens)
			playSoundWait(onesPCM[num%20],SND_HIGH,onesSize[num%20]);
		
		if((tens!=1) && ones)
			playSoundWait(onesPCM[ones],SND_HIGH,onesSize[ones]);
	}
	
	delay_ms(10);
}

void speakDice(int count, DieType die)
{
	output(count);
	speak(count);
	delay_ms(50);
	playSoundWait(D,SND_HIGH,sizeof(D));
	output(DieSides[die]);
	delay_ms(50);
	speak(DieSides[die]);
	output(BLANK_OUTPUT);
}

// Program Entry Point ********************************************************
//*****************************************************************************
int main(void)
{
	unsigned int seed = 0, seeded = 0, i;
	int          count = 0, result = 0, timer = 0;
	DieType      die = D20;
	State        state = State_Init;
	
	initIO();
	// Enable interrupts ******************************************************
	sei();

	delay_ms(500);
	
	count = 1;
	die = D20;
	speakDice(count,die);

	// Forever
	while (1)
	{
		uint8_t input;
		
		// Increment the seed counter
		++seed;
		
		// Implement the states of the application
		switch(state)
		{
			// Initialization
			case State_Init:
				// Initialize values
				result = 0;
				timer = 0;
				if(chkInput())
				{
					timer = DISPLAY_BLANK_TIME;
					state = State_Update_Dice;
				}
				break;
			// Idle
			case State_Idle:
				// If there is an active input flag...
				if(chkInput())
				{
					timer = DISPLAY_BLANK_TIME;
					state = State_Update_Dice;
				}
				// Else if the roll button has been pushed and the RNG has been seeded...
				else if(seeded)
				{
					// Configure the sleep mode to "Extended Standby"
					SMCR = SLEEP_MODE_EXT_STANDBY | 0x01;
					__asm__ __volatile__ ( "sleep" "\n\t" :: );
					SMCR = 0;
				}
				break;
			// Set the dice type to be rolled
			case State_Update_Dice:
				input = getInput();
				if(!--timer)
				 {
					state = State_Idle;
//					state = State_Speak_Dice;
					output(BLANK_OUTPUT);
				 }
				else if(input)
				{
					// Turn off the result indicator LED
					resultIndicator(false);
					
					// If the roll button is pushed...
					if(input & INPUT_ROLL)
					{
						state = State_Roll;
					}
					// Else if the die count is increased...
					else if(input & INPUT_COUNT_UP)
					{
						if(count<20)
							++count;
						output(count);
					}
					// Else if the die count is decreased...
					else if(input & INPUT_COUNT_DOWN)
					{
						if(count>1)
							--count;
						output(count);
					}
					// Else if the die count button is pushed...
					else if(input & INPUT_COUNT_BUTTON)
					{
						count = 1;
						output(count);
					}
					// Else if the die type is increased...
					else if(input & INPUT_TYPE_UP)
					{
						if(die<(DIE_TYPES-1))
							++die;
						output(DieSides[die]);
					}
					// Else if the die type is decreased...
					else if(input & INPUT_TYPE_DOWN)
					{
						if(die)
							--die;
						output(DieSides[die]);
					}
					// Else if the die type button is pushed...
					else if(input & INPUT_TYPE_BUTTON)
					{
						die = 6;
						output(DieSides[die]);
					}

					// Set the timer to display the updated value for the appropriate time
					timer = DISPLAY_BLANK_TIME;
				}
				delay_ms(1);
				break;
			// Momentarily display the dice type
			case State_Speak_Dice:
				speakDice(count,die);
				state = State_Idle;
				break;
			// Simulate a roll by flashing random values of the dice type on the display
			case State_Roll:
				// Speak the dice count and type
				speakDice(count,die);
				state = State_Roll_Result;
				break;
			// Calculate and display the roll result
			case State_Roll_Result:
				// If not seeded...
				if(!seeded)
				{
					// Seed the rand() function with the seed counter
					// that has been incrementing since startup
					srand(seed);
					seeded = 1;
				}
				result = 0;
				int save_count = count;
				int crit;
				do 
				{
					// Zero the crit result count
					crit = 0;
					// Play rolling dice sound
					playSoundWait(roll,SND_HIGH,sizeof(roll));
					for(i=0;i<count;++i)
					{
						int die_result;
						// Roll a single die
						die_result = (rand()%DieSides[die])+1;
						// Was this a "crit"?
						if(die_result == DieSides[die])
							++crit;
						// Sum up the total
						result += die_result;
					}
					// If exploding dice is enabled...
					if(getExplodingDice())
						count = crit;
					else
						crit = 0;
				} while (crit>0);
				count = save_count;

				resultIndicator(true);
				output(result);
				speak(result);
				// To debounce the roll button, clear the input register
				clrInput();
				state = State_Idle;
				break;
		}
		// Insert delay based on the current state
		// Note: if init or idle state, no delay inserted and
		// The seed counter increments as fast as this loop spins.
		//if(state>State_Idle)
		//	delay_ms(1);
	}
}
